@@ -8,6 +8,7 @@ from gradio.inputs import InputComponent
 from gradio.outputs import OutputComponent
 from gradio import networking, strings, utils
 from gradio.interpretation import quantify_difference_in_label
+from gradio.external import load_interface
 from gradio import encryptor
 import pkg_resources
 import requests
@@ -23,6 +24,7 @@ import os
 import copy
 import markdown2
 import json
+import csv
 from getpass import getpass
 
 analytics.write_key = "uxIFddIEuuUcFLf9VgH2teTEtPlWdkNy"
@@ -45,14 +47,30 @@ class Interface:
         return list(
             Interface.instances)
 
-    def __init__(self, fn, inputs, outputs, verbose=False, examples=None,
+    @classmethod
+    def load(cls, name, src=None, api_key=None, alias=None, **kwargs):
+        """
+        Loads a model and Interface from an external source repo
+        Parameters: 
+        name (str): the name of the model (e.g. "gpt2"), can include the `src` as prefix (e.g. "huggingface/gpt2")
+        src (str): the source of the model: `huggingface` or `gradio` (or empty if source is provided as a prefix in `name`)
+        api_key (str): optional api key for use with Hugging Face Model Hub
+        alias (str): optional, used as the name of the loaded model instead of the default name
+        Returns:
+        (Interface): an Interface object for the given model
+        """
+        interface_info = load_interface(name, src, api_key, alias)
+        interface_info.update(kwargs)
+        return cls(**interface_info)
+
+    def __init__(self, fn, inputs=None, outputs=None, verbose=False, examples=None,
                  examples_per_page=10, live=False,
                  layout="horizontal", show_input=True, show_output=True,
-                 capture_session=False, interpretation=None,
+                 capture_session=False, interpretation=None, theme="default", repeat_outputs_per_model=True,
                  title=None, description=None, article=None, thumbnail=None, 
-                 css=None, server_port=None, server_name=networking.LOCALHOST_NAME,
+                 css=None, server_port=None, server_name=networking.LOCALHOST_NAME, height=500, width=900,
                  allow_screenshot=True, allow_flagging=True, flagging_options=None, encrypt=False,
-                 show_tips=True, embedding=None, flagging_dir="flagged", analytics_enabled=True):
+                 show_tips=False, embedding=None, flagging_dir="flagged", analytics_enabled=True):
 
         """
         Parameters:
@@ -70,6 +88,7 @@ class Interface:
         description (str): a description for the interface; if provided, appears above the input and output components.
         article (str): an expanded article explaining the interface; if provided, appears below the input and output components. Accepts Markdown and HTML content.
         thumbnail (str): path to image or src to use as display picture for models listed in gradio.app/hub
+        theme (str): Theme to use - one of "dark", "default", or "huggingface".
         css (str): custom css or path to custom css file to use with interface.
         server_port (int): will start gradio app on this port (if available) 
         server_name (str): to make app accessible on local network set to "0.0.0.0".
@@ -80,7 +99,6 @@ class Interface:
         flagging_dir (str): what to name the dir where flagged data is stored.
         show_tips (bool): if True, will occasionally show tips about new Gradio features
         """
-
         def get_input_instance(iface):
             if isinstance(iface, str):
                 shortcut = InputComponent.get_all_shortcut_implementations()[iface]
@@ -89,7 +107,7 @@ class Interface:
                 return iface
             else:
                 raise ValueError("Input interface must be of type `str` or "
-                                 "`InputComponent`")
+                                 "`InputComponent` but is {}".format(iface))
 
         def get_output_instance(iface):
             if isinstance(iface, str):
@@ -103,20 +121,22 @@ class Interface:
                     "`OutputComponent`"
                 )
 
-        if isinstance(inputs, list):
-            self.input_interfaces = [get_input_instance(i) for i in inputs]
-        else:
-            self.input_interfaces = [get_input_instance(inputs)]
-        if isinstance(outputs, list):
-            self.output_interfaces = [get_output_instance(i) for i in outputs]
-        else:
-            self.output_interfaces = [get_output_instance(outputs)]
         if not isinstance(fn, list):
             fn = [fn]
+        if isinstance(inputs, list):
+            self.input_components = [get_input_instance(i) for i in inputs]
+        else:
+            self.input_components = [get_input_instance(inputs)]
+        if isinstance(outputs, list):
+            self.output_components = [get_output_instance(i) for i in outputs]
+        else:
+            self.output_components = [get_output_instance(outputs)]
 
-        self.output_interfaces *= len(fn)
+        if repeat_outputs_per_model:
+            self.output_components *= len(fn)
         self.predict = fn
         self.function_names = [func.__name__ for func in fn]
+        self.__name__ = ", ".join(self.function_names)
         self.verbose = verbose
         self.status = "OFF"
         self.live = live
@@ -135,6 +155,9 @@ class Interface:
             article = markdown2.markdown(article)
         self.article = article
         self.thumbnail = thumbnail
+        self.theme = theme
+        self.height = height
+        self.width = width
         if css is not None and os.path.exists(css):
             with open(css) as css_file:
                 self.css = css_file.read()
@@ -193,14 +216,31 @@ class Interface:
             except (requests.ConnectionError, requests.exceptions.ReadTimeout):
                 pass  # do not push analytics if no network
 
+    def __call__(self, params_per_function):
+        return self.predict[0](params_per_function)
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        repr = "Gradio Interface for: {}".format(", ".join(fn.__name__ for fn in self.predict))
+        repr += "\n" + "-"*len(repr)
+        repr += "\ninputs:"
+        for component in self.input_components:
+            repr += "\n|-{}".format(str(component))
+        repr += "\noutputs:"
+        for component in self.output_components:
+            repr+= "\n|-{}".format(str(component))
+        return repr
+        
     def get_config_file(self):
         config = {
-            "input_interfaces": [
-                (iface.__class__.__name__.lower(), iface.get_template_context())
-                for iface in self.input_interfaces],
-            "output_interfaces": [
-                (iface.__class__.__name__.lower(), iface.get_template_context())
-                for iface in self.output_interfaces],
+            "input_components": [
+                iface.get_template_context()
+                for iface in self.input_components],
+            "output_components": [
+                iface.get_template_context()
+                for iface in self.output_components],
             "function_count": len(self.predict),
             "live": self.live,
             "examples_per_page": self.examples_per_page,
@@ -210,6 +250,7 @@ class Interface:
             "title": self.title,
             "description": self.description,
             "article": self.article,
+            "theme": self.theme,
             "thumbnail": self.thumbnail,
             "allow_screenshot": self.allow_screenshot,
             "allow_flagging": self.allow_flagging,
@@ -219,23 +260,42 @@ class Interface:
         }
         try:
             param_names = inspect.getfullargspec(self.predict[0])[0]
-            for iface, param in zip(config["input_interfaces"], param_names):
-                if not iface[1]["label"]:
-                    iface[1]["label"] = param.replace("_", " ")
-            for i, iface in enumerate(config["output_interfaces"]):
-                outputs_per_function = int(len(self.output_interfaces) / len(self.predict))
+            for iface, param in zip(config["input_components"], param_names):
+                if not iface["label"]:
+                    iface["label"] = param.replace("_", " ")
+            for i, iface in enumerate(config["output_components"]):
+                outputs_per_function = int(len(self.output_components) / len(self.predict))
                 function_index = i // outputs_per_function
                 component_index = i - function_index * outputs_per_function
                 ret_name = "Output " + str(component_index + 1) if outputs_per_function > 1 else "Output"
-                if not iface[1]["label"]:
-                    iface[1]["label"] = ret_name
+                if not iface["label"]:
+                    iface["label"] = ret_name
                 if len(self.predict) > 1:
-                    iface[1]["label"] = self.function_names[function_index].replace("_", " ") + ": " + iface[1]["label"]
+                    iface["label"] = self.function_names[function_index].replace("_", " ") + ": " + iface["label"]
                     
         except ValueError:
             pass
-        if self.examples is not None and not isinstance(self.examples, str):
-            config["examples"] = self.examples
+        if self.examples is not None:
+            if isinstance(self.examples, str):
+                if not os.path.exists(self.examples):
+                    raise FileNotFoundError("Could not find examples directory: " + self.examples)
+                log_file = os.path.join(self.examples, "log.csv")
+                if not os.path.exists(log_file):
+                    if len(self.input_components) == 1:
+                        examples = [[item] for item in os.listdir(self.examples)]
+                    else:
+                        raise FileNotFoundError("Could not find log file (required for multiple inputs): " + log_file)
+                else:
+                    with open(log_file) as logs:
+                        examples = list(csv.reader(logs)) 
+                        examples = examples[1:] #remove header
+                for i, example in enumerate(examples):
+                    for j, (interface, cell) in enumerate(zip(self.input_components + self.output_components, example)):
+                        examples[i][j] = interface.restore_flagged(cell)
+                config["examples"] = examples
+                config["examples_dir"] = self.examples
+            else:
+                config["examples"] = self.examples
         return config
 
     def run_prediction(self, processed_input, return_duration=False):
@@ -257,8 +317,9 @@ class Interface:
                         raise exception
             duration = time.time() - start
 
-            if len(self.output_interfaces) == len(self.predict):
+            if len(self.output_components) == len(self.predict):
                 prediction = [prediction]
+            
             durations.append(duration)
             predictions.extend(prediction)
         
@@ -273,17 +334,17 @@ class Interface:
         processed output: a list of processed  outputs to return as the prediction(s).
         duration: a list of time deltas measuring inference time for each prediction fn.
         """
-        processed_input = [input_interface.preprocess(raw_input[i])
-                           for i, input_interface in enumerate(self.input_interfaces)]
+        processed_input = [input_component.preprocess(raw_input[i])
+                           for i, input_component in enumerate(self.input_components)]
         predictions, durations = self.run_prediction(processed_input, return_duration=True)
-        processed_output = [output_interface.postprocess(
-            predictions[i]) for i, output_interface in enumerate(self.output_interfaces)]
+        processed_output = [output_component.postprocess(
+            predictions[i]) for i, output_component in enumerate(self.output_components)]
         return processed_output, durations
     
     def embed(self, processed_input):
         if self.embedding == "default":
-            embeddings = np.concatenate([input_interface.embed(processed_input[i])
-                            for i, input_interface in enumerate(self.input_interfaces)])
+            embeddings = np.concatenate([input_component.embed(processed_input[i])
+                            for i, input_component in enumerate(self.input_components)])
         else:
             embeddings = self.embedding(*processed_input)
         return embeddings
@@ -295,23 +356,23 @@ class Interface:
         :param raw_input: a list of raw inputs to apply the interpretation(s) on.
         """
         if self.interpretation == "default":
-            processed_input = [input_interface.preprocess(raw_input[i])
-                            for i, input_interface in enumerate(self.input_interfaces)]
+            processed_input = [input_component.preprocess(raw_input[i])
+                            for i, input_component in enumerate(self.input_components)]
             original_output = self.run_prediction(processed_input)
             scores, alternative_outputs = [], []
             for i, x in enumerate(raw_input):
-                input_interface = self.input_interfaces[i]
+                input_component = self.input_components[i]
                 neighbor_raw_input = list(raw_input)
-                neighbor_values, interpret_kwargs, interpret_by_removal = input_interface.get_interpretation_neighbors(x)
+                neighbor_values, interpret_kwargs, interpret_by_removal = input_component.get_interpretation_neighbors(x)
                 interface_scores = []
                 alternative_output = []
                 for neighbor_input in neighbor_values:
                     neighbor_raw_input[i] = neighbor_input
-                    processed_neighbor_input = [input_interface.preprocess(neighbor_raw_input[i])
-                                    for i, input_interface in enumerate(self.input_interfaces)]
+                    processed_neighbor_input = [input_component.preprocess(neighbor_raw_input[i])
+                                    for i, input_component in enumerate(self.input_components)]
                     neighbor_output = self.run_prediction(processed_neighbor_input)
-                    processed_neighbor_output = [output_interface.postprocess(
-                        neighbor_output[i]) for i, output_interface in enumerate(self.output_interfaces)]
+                    processed_neighbor_output = [output_component.postprocess(
+                        neighbor_output[i]) for i, output_component in enumerate(self.output_components)]
 
                     alternative_output.append(processed_neighbor_output)
                     interface_scores.append(quantify_difference_in_label(self, original_output, neighbor_output))
@@ -319,12 +380,12 @@ class Interface:
                 if not interpret_by_removal:
                     interface_scores = [-score for score in interface_scores]
                 scores.append(
-                    input_interface.get_interpretation_scores(
+                    input_component.get_interpretation_scores(
                         raw_input[i], neighbor_values, interface_scores, **interpret_kwargs))
             return scores, alternative_outputs
         else:
-            processed_input = [input_interface.preprocess(raw_input[i])
-                               for i, input_interface in enumerate(self.input_interfaces)]
+            processed_input = [input_component.preprocess(raw_input[i])
+                               for i, input_component in enumerate(self.input_components)]
             interpreter = self.interpretation
 
             if self.capture_session and self.session is not None:
@@ -362,18 +423,18 @@ class Interface:
             print("Test launch: {}()...".format(predict_fn.__name__), end=' ')
 
             raw_input = []
-            for input_interface in self.input_interfaces:
-                if input_interface.test_input is None:  # If no test input is defined for that input interface
+            for input_component in self.input_components:
+                if input_component.test_input is None:  # If no test input is defined for that input interface
                     print("SKIPPED")
                     break
                 else:  # If a test input is defined for each interface object
-                    raw_input.append(input_interface.test_input)
+                    raw_input.append(input_component.test_input)
             else:
                 self.process(raw_input)
                 print("PASSED")
                 continue
 
-    def launch(self, inline=None, inbrowser=None, share=False, debug=False, auth=None, auth_message=None, private_endpoint=None):
+    def launch(self, inline=None, inbrowser=None, share=False, debug=False, auth=None, auth_message=None, private_endpoint=None, prevent_thread_lock=False):
         """
         Parameters:
         inline (bool): whether to display in the interface inline on python notebooks.
@@ -463,9 +524,9 @@ class Interface:
                 if share:
                     while not networking.url_ok(share_url):
                         time.sleep(1)
-                    display(IFrame(share_url, width=1000, height=500))
+                    display(IFrame(share_url, width=self.width, height=self.height))
                 else:
-                    display(IFrame(path_to_local_server, width=1000, height=500))
+                    display(IFrame(path_to_local_server, width=self.width, height=self.height))
             except ImportError:
                 pass  # IPython is not available so does not print inline.
 
@@ -480,9 +541,10 @@ class Interface:
                 sys.stdout.flush()
                 time.sleep(0.1)
         is_in_interactive_mode = bool(getattr(sys, 'ps1', sys.flags.interactive))
-        if not is_in_interactive_mode:
+        if not prevent_thread_lock and not is_in_interactive_mode:
             self.run_until_interrupted(thread, path_to_local_server)
-        
+
+
         return app, path_to_local_server, share_url
 
 
@@ -493,6 +555,7 @@ class Interface:
         else:
             comet_ml.log_text(self.local_url)
             comet_ml.end()
+    
 
 def show_tip(io):
     if not(io.show_tips):
