@@ -8,11 +8,11 @@ def diff(original, perturbed):
     return score
 
 def quantify_difference_in_label(interface, original_output, perturbed_output):
-    output_interface = interface.output_interfaces[0]
-    post_original_output = output_interface.postprocess(original_output[0])
-    post_perturbed_output = output_interface.postprocess(perturbed_output[0])
+    output_component = interface.output_components[0]
+    post_original_output = output_component.postprocess(original_output[0])
+    post_perturbed_output = output_component.postprocess(perturbed_output[0])
 
-    if type(output_interface) == Label:
+    if type(output_component) == Label:
         original_label = post_original_output["label"]
         perturbed_label = post_perturbed_output["label"]
 
@@ -24,7 +24,7 @@ def quantify_difference_in_label(interface, original_output, perturbed_output):
         else:
             score = diff(original_label, perturbed_label)
         return score
-    elif type(output_interface) == Textbox:
+    elif type(output_component) == Textbox:
         score = diff(post_original_output, post_perturbed_output)
         return score
 
